﻿using System;
using System.Windows.Controls;

using Buddy.Wildstar.Engine;
using Buddy.Wildstar.Game;
using Buddy.Wildstar.Game.Actors;
using Buddy.Wildstar.BotCommon;

using log4net;
using MahApps.Metro.Controls;
using System.IO;
using System.Windows.Markup;


namespace CombatBotEx {

    public class CombatBotEx : IBot, IUIButtonProvider {

        private ILog Log = LogManager.GetLogger("CombatBotEx");

        #region IAuthored Implementation
        public string Author { get { return "Paulmuadip"; } }
        public string Name { get { return "CombatBotEx"; } }
        public string Version { get { return "1.0.4"; } }
        #endregion

        #region IBot Implementation

        private bool _pulseMe = false;

        public void OnRegistered() { throw new NotImplementedException(); }
        public void OnUnregistered() { throw new NotImplementedException(); }

        public void OnInitialize() {
            Log.Debug("OnInitialize");
            CombatBotExConfig.Instance.SettingsVersion = string.Format("Version {0} by {1}", this.Version, this.Author);
            _pulseMe = false;
        }

        public void OnUninitialize() {
            Log.Debug("OnUninitialize");
            _pulseMe = false;
        }

        public void OnStart() {
            Log.Info("Bot started.");
            _pulseMe = true;
            GameEngine.BotPulsator.RegisterForPulse(GameManager.Lua.Events);
            GameEngine.BotPulsator.RegisterForPulse(GameEngine.CurrentRoutine);
            SpellManager.SetSpellCastingBehavior(SpellCastingBehavior.Normal);
        }

        public void OnStop() {
            Log.Info("Bot stopped.");
            _pulseMe = false;
            if (GameEngine.CurrentRoutine != null) {
                GameEngine.BotPulsator.UnregisterForPulse(GameEngine.CurrentRoutine);
            }
            GameEngine.BotPulsator.UnregisterForPulse(GameManager.Lua.Events);
        }


        public bool CanBePulsed { get { return _pulseMe; } }

        public bool Equals(IBot other) { throw new NotImplementedException(); }

        #endregion

        #region IBot : IPulsable.Pulse() Implementation

        // Todo: move to be configurable
        private int throttleDelay = 1; // delay in seconds for considering actions during Pulse()
        private int collectibleRadius = 5; // how far to look for collectibles
        private int maxLevelDifference = 3; // how many levels above player's level to auto-attack
        private double minPlayerHealthPercent = 0.5; // minimum player health percentage required for auto-attack

        private DateTime throttle = DateTime.Now;

        public void Pulse() {

            #region Throttle - Skip pulses
            // Only evaluate actions every 'throttleDelay' seconds.  Throttle activities.
            if ((DateTime.Now - this.throttle).TotalSeconds < this.throttleDelay) return;
            this.throttle = DateTime.Now;
            #endregion

            #region Actions
            // Try to loot collectibles or loot
            if (CombatBotExConfig.Instance.SettingsAutoLoot) VacuumLoot();
            if (CombatBotExConfig.Instance.SettingsAutoCollect) LootCollectibles();
            if (CombatBotExConfig.Instance.SettingsAutoAttack) EngageTarget();
            #endregion
        }

        #endregion

        #region EngageTarget Implementation

        private SpellManager.SpellInformation attackSpell = SpellManager.GetMiscSpell(MiscSpellType.RightClickAttack);

        public void EngageTarget() {

            // Make sure there's a valid me and a valid target
            Actor me = GameManager.LocalPlayer;
            if (me == null) return;
            Actor target = me.CurrentTarget;
            if (target == null) return;
            CreatureInfo info = target.CreatureInfo;
            if (info == null) return;

            if (CombatBotExConfig.Instance.SettingsCheckLevel) {
                // Don't auto-attack if target too high level
                int levelDifference = (int)target.Level - (int)me.Level; // positive implies target is of greater level than player
                if (levelDifference > maxLevelDifference) {
                    Log.Warn(info.Name + " [" + target.Level + "] is too high level. Not auto-engaging.");
                    return;
                }
            }

            if (CombatBotExConfig.Instance.SettingsCheckHealth) {
                // Abort engaging target if player health below minimum
                if (me.HealthPercent <= minPlayerHealthPercent)
                    return;
            }

            // Sanity check target
            if (target.IsAlive && (!me.IsInCombat) && (info.CanBeAttacked)) {
                Log.Info("Attacking " + info.Name + " with " + me.HealthPercent + "% health");
                if (CombatBotExConfig.Instance.SettingsAutoFace) target.Face();
                this.attackSpell.Cast();
            }
        }

        #endregion

        #region Settings Dialog Implementation and Handling

        public string ButtonText { get { return "CombatBotEx"; } }

        public UserControl Control {
            get {
                Log.Info("Loading XAML");
                using (var fs = new FileStream(@"Bots\CombatBotEx\CombatBotExGui.xaml", FileMode.Open)) {
                    var root = (UserControl)XamlReader.Load(fs);
                    root.DataContext = CombatBotExConfig.Instance; // WPF DataBinding, bind it to our CombatBotExConfig instance
                    return root;
                };
            }
        }

        public void OnButtonClicked(object sender) {
            Log.Info("Opening Configuration Dialog");
            MetroNavigationWindow configWindow = new MetroNavigationWindow() {
                Title = "CombatBotEx Settings",
                Content = this.Control,
                ShowCloseButton = false,
                ShowMinButton = false,
                ShowMaxRestoreButton = false,
                ShowIconOnTitleBar = false,
                SizeToContent = System.Windows.SizeToContent.WidthAndHeight
            };
            configWindow.ShowDialog();
        }

        #endregion

        #region Looting Implementations

        public void LootCollectibles() {
            // Check if there are any collectibles (C) nearby
            Buddy.Common.DefaultDictionary<uint, Actor> objects = GameManager.Actors;
            foreach (var o in objects.Values) {
                if ((o.Distance < collectibleRadius) && (o.IsCollect)) {
                    Actor me = GameManager.LocalPlayer;
                    if ((!me.IsMoving) && (!me.IsOnTaxi) && (!me.IsBusy) && (!me.IsCasting)) {
                        Log.Info("Collecting " + o.Name);
                        o.Interact();
                    }
                }
            }
        }

        private SpellManager.SpellInformation lootSpell = SpellManager.GetMiscSpell(MiscSpellType.VacuumLoot);
        public void VacuumLoot() {
            // Check if there is anything to loot (V)
            if (GameManager.CanVacuum) {
                Log.Info("Looting");
                CommonBehaviors.VacuumLoot();
            }
        }

        #endregion
    }
}
